import time
from typing import List, Tuple

from prometheus_client import Counter, Histogram
from prometheus_client.metrics import MetricWrapperBase


class MetricsName:
    LATENCY = 'RQS_LATENCY'
    COUNT = 'RQS_COUNT'


def init(app, latency_buckets=None, multiprocess_mode='all',
         metrics_list: List[Tuple[str, MetricWrapperBase]] = None):
    app.metrics[MetricsName.COUNT] = Counter(
        'sanic_request_count',
        'Sanic Request Count',
        ['method', 'endpoint', 'http_status']
    )

    hist_kwargs = {}
    if latency_buckets is not None:
        hist_kwargs = {'buckets': latency_buckets}
    app.metrics[MetricsName.LATENCY] = Histogram(
        'sanic_request_latency_sec',
        'Sanic Request Latency Histogram',
        ['method', 'endpoint', 'http_status'],
        **hist_kwargs
    )

    if metrics_list:
        for name, pm_metric in metrics_list:
            # todo set multiprocess_mode is Gauge
            # app.metrics[name'] = Gauge(
            #             ...
            #             multiprocess_mode=multiprocess_mode
            #         )
            app.metrics[name] = pm_metric


def before_request_handler(request):
    request['__START_TIME__'] = time.time()


def after_request_handler(request, response, get_endpoint_fn):
    lat = time.time() - request['__START_TIME__']
    endpoint = get_endpoint_fn(request)

    # Note, that some handlers can ignore response logic,
    # for example, websocket handler
    response_status = response.status if response else 200
    request.app.metrics[MetricsName.LATENCY].labels(
        request.method, endpoint, response_status
    ).observe(lat)
    request.app.metrics[MetricsName.COUNT].labels(
        request.method, endpoint, response_status
    ).inc()



from setuptools import setup

from scripts.release import get_version

setup(
    name='prometheus-sanic',
    version=f'{get_version()}',
    description='Exposes Prometheus monitoring metrics of Sanic apps.',
    url='https://github.com/skar404/prometheus-sanic',
    author='user',
    author_email='skar404@gmail.com',
    license='MIT',
    packages=['prometheus_sanic'],
    zip_safe=False,
    platforms='any',
    install_requires=[
        'sanic>=18.12',
        'prometheus-client~=0.7.1',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',

        'Operating System :: OS Independent',
        'Environment :: Web Environment',
        'Topic :: System :: Monitoring',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='sanic prometheus monitoring'
)
